@@ -262,21 +262,34 @@ fn heap_location() -> u64 {
     let mut p = HEAP_LOCATION.load(std::sync::atomic::Ordering::Relaxed) as u64;
     if p == 0 {
         let ptr = allocate_ptr();
-        HEAP_LOCATION.store(ptr, std::sync::atomic::Ordering::Relaxed);
-        p = ptr as u64;
+        p = match HEAP_LOCATION.compare_exchange(
+            std::ptr::null_mut(),
+            ptr,
+            std::sync::atomic::Ordering::Relaxed,
+            std::sync::atomic::Ordering::Relaxed,
+        ) {
+            Ok(_) => {
+                ptr as u64
+            },
+            Err(ptr) => {
+                println!("race, ptr is {:p}", ptr);
+                ptr as u64
+            }
+        }
     }
     p
 }
 #[cfg(feature = "tinyset")]
-fn sz<T>() -> u64 {
+const fn sz<T>() -> u64 {
     std::mem::align_of::<T>() as u64
 }
-/// The `Fits64` implementation for `Intern<T>` is designed to
-/// normally give (relatively) small numbers, by XORing with a fixed
-/// pointer that is also on the heap.  This should make the most
-/// significant bits of the resulting u64 be zero, which will mean
-/// that `Set64` (which is space-efficient in storing small integers)
-/// can store this result in fewer than 8 bytes.
+/// The `Fits64` implementation for `Intern<T>` is designed to normally give
+/// (relatively) small numbers, by XORing with a fixed pointer that is also on
+/// the heap.  The pointer is also divided by its alignment to eliminate those
+/// redundant insignificant zeros.  This should make the most significant bits
+/// of the resulting u64 be zero, which will mean that `Set64` (which is
+/// space-efficient in storing small integers) can store this result in far
+/// fewer than 8 bytes.
 #[cfg(feature = "tinyset")]
 impl<T: Debug> Fits64 for Intern<T> {
     unsafe fn from_u64(x: u64) -> Self {
@@ -298,7 +311,9 @@ impl<T: Debug> Fits64 for Intern<T> {
 impl<T: Debug> Fits64 for LocalIntern<T> {
     unsafe fn from_u64(x: u64) -> Self {
         LocalIntern {
-            pointer: std::ptr::NonNull::new_unchecked(((x ^ heap_location() / sz::<T>()) * sz::<T>()) as *mut T),
+            pointer: std::ptr::NonNull::new_unchecked(
+                ((x ^ heap_location() / sz::<T>()) * sz::<T>()) as *mut T,
+            ),
         }
     }
     fn to_u64(self) -> u64 {
